"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const validLambdaTriggers = [
    'viewer-request',
    'origin-request',
    'origin-response',
    'viewer-response',
];
const triggersAllowedBody = ['viewer-request', 'origin-request'];
const makeCacheItem = (eventType, lambdaConfig) => {
    let arn, includeBody;
    if (typeof lambdaConfig === 'string') {
        arn = lambdaConfig;
        includeBody = triggersAllowedBody.includes(eventType);
    }
    else {
        ({ arn, includeBody } = lambdaConfig);
        if (includeBody && !triggersAllowedBody.includes(eventType)) {
            throw new Error(`"includeBody" not allowed for ${eventType} lambda triggers.`);
        }
    }
    return {
        EventType: eventType,
        LambdaFunctionARN: arn,
        IncludeBody: includeBody,
    };
};
const addLambdaAtEdgeToCacheBehavior = (cacheBehavior, lambdaAtEdge = {}) => {
    Object.keys(lambdaAtEdge).forEach((eventType) => {
        var _a;
        if (!validLambdaTriggers.includes(eventType)) {
            throw new Error(`"${eventType}" is not a valid lambda trigger. See https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/lambda-cloudfront-trigger-events.html for valid event types.`);
        }
        const haveValidConfig = typeof lambdaAtEdge[eventType] === 'string' || ((_a = lambdaAtEdge[eventType]) === null || _a === void 0 ? void 0 : _a.arn);
        if (haveValidConfig) {
            cacheBehavior.LambdaFunctionAssociations.Items.push(makeCacheItem(eventType, lambdaAtEdge[eventType]));
            cacheBehavior.LambdaFunctionAssociations.Quantity =
                cacheBehavior.LambdaFunctionAssociations.Quantity + 1;
        }
    });
};
exports.default = addLambdaAtEdgeToCacheBehavior;
