"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = __importDefault(require("url"));
const getOriginConfig = (origin, { originAccessIdentityId = '' }) => {
    const originUrl = typeof origin === 'string' ? origin : origin.url;
    const protocolPolicy = typeof origin === 'string' ? null : origin.protocolPolicy;
    const { hostname, pathname } = url_1.default.parse(originUrl);
    const originConfig = {
        Id: `${hostname}${pathname}`.replace(/\/$/, ''),
        DomainName: hostname,
        CustomHeaders: {
            Quantity: 0,
            Items: [],
        },
        OriginPath: pathname === '/' ? '' : pathname,
    };
    if (originUrl.includes('s3') && !originUrl.includes('s3-website')) {
        const bucketName = hostname.split('.')[0];
        originConfig.Id = pathname === '/' ? bucketName : `${bucketName}${pathname}`;
        originConfig.DomainName = hostname;
        originConfig.S3OriginConfig = {
            OriginAccessIdentity: originAccessIdentityId
                ? `origin-access-identity/cloudfront/${originAccessIdentityId}`
                : '',
        };
    }
    else {
        originConfig.CustomOriginConfig = {
            HTTPPort: 80,
            HTTPSPort: 443,
            OriginProtocolPolicy: protocolPolicy || 'https-only',
            OriginSslProtocols: {
                Quantity: 1,
                Items: ['TLSv1.2'],
            },
            OriginReadTimeout: 30,
            OriginKeepaliveTimeout: 5,
        };
    }
    return originConfig;
};
exports.default = getOriginConfig;
