"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.inputsChanged = exports.updateAssumeRolePolicy = exports.getRole = exports.deleteRole = exports.createRole = exports.removeRolePolicy = exports.addRolePolicy = void 0;
const core_1 = require("@serverless/core");
const ramda_1 = require("ramda");
exports.addRolePolicy = async ({ iam, name, policy, }) => {
    if (ramda_1.has('arn', policy)) {
        await iam
            .attachRolePolicy({
            RoleName: name,
            PolicyArn: policy.arn,
        })
            .promise();
    }
    else if (!ramda_1.isEmpty(policy)) {
        await iam
            .putRolePolicy({
            RoleName: name,
            PolicyName: `${name}-policy`,
            PolicyDocument: JSON.stringify(policy),
        })
            .promise();
    }
    return core_1.utils.sleep(15000);
};
exports.removeRolePolicy = async ({ iam, name, policy, }) => {
    if (ramda_1.has('arn', policy)) {
        await iam
            .detachRolePolicy({
            RoleName: name,
            PolicyArn: policy.arn,
        })
            .promise();
    }
    else if (!ramda_1.isEmpty(policy)) {
        await iam
            .deleteRolePolicy({
            RoleName: name,
            PolicyName: `${name}-policy`,
        })
            .promise();
    }
};
exports.createRole = async ({ iam, name, service, policy, }) => {
    const assumeRolePolicyDocument = {
        Version: '2012-10-17',
        Statement: {
            Effect: 'Allow',
            Principal: {
                Service: service,
            },
            Action: 'sts:AssumeRole',
        },
    };
    const roleRes = await iam
        .createRole({
        RoleName: name,
        Path: '/',
        AssumeRolePolicyDocument: JSON.stringify(assumeRolePolicyDocument),
    })
        .promise();
    await exports.addRolePolicy({
        iam,
        name,
        policy,
    });
    return roleRes.Role.Arn;
};
exports.deleteRole = async ({ iam, name, policy, }) => {
    try {
        await exports.removeRolePolicy({
            iam,
            name,
            policy,
        });
        await iam
            .deleteRole({
            RoleName: name,
        })
            .promise();
    }
    catch (error) {
        if (error.message !== `Policy ${policy.arn} was not found.` && error.code !== 'NoSuchEntity') {
            throw error;
        }
    }
};
exports.getRole = async ({ iam, name, }) => {
    try {
        const res = await iam.getRole({ RoleName: name }).promise();
        return {
            name: res.Role.RoleName,
            arn: res.Role.Arn,
            service: JSON.parse(decodeURIComponent(res.Role.AssumeRolePolicyDocument))
                .Statement[0].Principal.Service,
        };
    }
    catch (e) {
        if (e.message.includes('cannot be found')) {
            return null;
        }
        throw e;
    }
};
exports.updateAssumeRolePolicy = async ({ iam, name, service, }) => {
    const assumeRolePolicyDocument = {
        Version: '2012-10-17',
        Statement: {
            Effect: 'Allow',
            Principal: {
                Service: service,
            },
            Action: 'sts:AssumeRole',
        },
    };
    await iam
        .updateAssumeRolePolicy({
        RoleName: name,
        PolicyDocument: JSON.stringify(assumeRolePolicyDocument),
    })
        .promise();
};
exports.inputsChanged = (prevRole, role) => {
    var _a, _b;
    const inputs = ramda_1.pick(['service'], role);
    const prevInputs = ramda_1.pick(['service'], prevRole);
    if (ramda_1.type(inputs.service) === 'Array') {
        (_a = inputs === null || inputs === void 0 ? void 0 : inputs.service) === null || _a === void 0 ? void 0 : _a.sort();
    }
    if (ramda_1.type(prevInputs.service) === 'Array') {
        (_b = prevInputs === null || prevInputs === void 0 ? void 0 : prevInputs.service) === null || _b === void 0 ? void 0 : _b.sort();
    }
    return ramda_1.not(ramda_1.equals(inputs, prevInputs));
};
