"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const aws_sdk_1 = require("aws-sdk");
const utils_1 = require("./utils");
exports.default = async ({ bucketName, credentials }) => {
    let s3 = new aws_sdk_1.S3({ ...credentials });
    try {
        const { Status } = await s3
            .getBucketAccelerateConfiguration({
            Bucket: bucketName,
        })
            .promise();
        if (Status === 'Enabled') {
            s3 = new aws_sdk_1.S3({ ...credentials, useAccelerateEndpoint: true });
        }
    }
    catch (err) {
        console.warn(`Checking for bucket acceleration failed, falling back to non-accelerated S3 client. Err: ${err.message}`);
    }
    return {
        uploadFile: async ({ filePath, cacheControl, s3Key, }) => {
            const fileBody = await fs_extra_1.readFile(filePath);
            return s3
                .upload({
                Bucket: bucketName,
                Key: s3Key || filePath,
                Body: fileBody,
                ContentType: utils_1.getMimeType(filePath),
                CacheControl: cacheControl || undefined,
            })
                .promise();
        },
        listFiles: async ({ s3Key }) => {
            return s3
                .listObjectsV2({
                Bucket: bucketName,
                Prefix: s3Key,
            })
                .promise();
        },
        downloadFile: async ({ s3Key }) => {
            return s3
                .getObject({
                Bucket: bucketName,
                Key: s3Key,
            })
                .promise();
        },
        setVersioning: async ({ versioned }) => {
            return s3
                .putBucketVersioning({
                Bucket: bucketName,
                VersioningConfiguration: {
                    Status: versioned ? 'Enabled' : 'Suspended',
                },
            })
                .promise();
        },
    };
};
