"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const fs_extra_1 = require("fs-extra");
const s3_1 = __importDefault(require("./s3"));
const utils_1 = require("./utils");
const STATE_ROOT = '.next-deploy';
const syncStageStateDirectory = async ({ name: stage, bucketName, credentials, nextConfigDir, syncTo, versioned, }) => {
    const s3 = await s3_1.default({
        bucketName,
        credentials,
    });
    const stateRootDirectory = path_1.default.join(nextConfigDir, STATE_ROOT);
    if (syncTo) {
        const stateRootDirectoryFiles = await utils_1.readDirectoryFiles(path_1.default.join(stateRootDirectory, stage));
        const buildStateRootDirectoryFilesUploads = stateRootDirectoryFiles
            .filter(utils_1.filterOutDirectories)
            .map(async (fileItem) => s3.uploadFile({
            s3Key: utils_1.pathToPosix(path_1.default.relative(path_1.default.resolve(nextConfigDir), fileItem.path)).replace(`${STATE_ROOT}/`, ''),
            filePath: fileItem.path,
        }));
        return Promise.all([...buildStateRootDirectoryFilesUploads]);
    }
    else {
        const files = [];
        if (versioned !== undefined) {
            await s3.setVersioning({ versioned });
        }
        const bucketFiles = await s3.listFiles({
            s3Key: stage,
        });
        for (const file of bucketFiles.Contents || []) {
            if (file.Key) {
                const fileData = await s3.downloadFile({ s3Key: file.Key });
                files.push({ name: path_1.default.join(STATE_ROOT, file.Key), data: fileData });
            }
        }
        for (const { name, data } of files || []) {
            if (data.Body) {
                await fs_extra_1.writeFile(name, data.Body.toString());
            }
        }
    }
};
exports.default = syncStageStateDirectory;
