"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IMMUTABLE_CACHE_CONTROL_HEADER = exports.SERVER_CACHE_CONTROL_HEADER = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = require("fs-extra");
const s3_1 = __importDefault(require("./s3"));
const utils_1 = require("./utils");
const getPublicAssetCacheControl_1 = __importDefault(require("./getPublicAssetCacheControl"));
exports.SERVER_CACHE_CONTROL_HEADER = 'public, max-age=0, s-maxage=2678400, must-revalidate';
exports.IMMUTABLE_CACHE_CONTROL_HEADER = 'public, max-age=31536000, immutable';
const uploadStaticAssets = async ({ bucketName, nextConfigDir, nextStaticDir = nextConfigDir, publicDirectoryCache, credentials, }) => {
    const s3 = await s3_1.default({
        bucketName,
        credentials,
    });
    const dotNextDirectory = path_1.default.join(nextConfigDir, '.next');
    const buildStaticFiles = await utils_1.readDirectoryFiles(path_1.default.join(dotNextDirectory, 'static'));
    const buildStaticFileUploads = buildStaticFiles
        .filter(utils_1.filterOutDirectories)
        .map(async (fileItem) => {
        const s3Key = utils_1.pathToPosix(path_1.default.relative(path_1.default.resolve(nextConfigDir), fileItem.path).replace(/^.next/, '_next'));
        return s3.uploadFile({
            s3Key,
            filePath: fileItem.path,
            cacheControl: exports.IMMUTABLE_CACHE_CONTROL_HEADER,
        });
    });
    const pagesManifest = await fs_extra_1.readJSON(path_1.default.join(dotNextDirectory, 'serverless/pages-manifest.json'));
    const htmlPageUploads = Object.values(pagesManifest)
        .filter((pageFile) => pageFile.endsWith('.html'))
        .map((relativePageFilePath) => {
        const pageFilePath = utils_1.pathToPosix(path_1.default.join(dotNextDirectory, `serverless/${relativePageFilePath}`));
        return s3.uploadFile({
            s3Key: `static-pages/${relativePageFilePath.replace(/^pages\//, '')}`,
            filePath: pageFilePath,
            cacheControl: exports.SERVER_CACHE_CONTROL_HEADER,
        });
    });
    const prerenderManifest = await fs_extra_1.readJSON(path_1.default.join(dotNextDirectory, 'prerender-manifest.json'));
    const prerenderManifestJSONPropFileUploads = Object.keys(prerenderManifest.routes).map((key) => {
        const pageFilePath = utils_1.pathToPosix(path_1.default.join(dotNextDirectory, `serverless/pages/${key.endsWith('/') ? `${key}index.json` : `${key}.json`}`));
        return s3.uploadFile({
            s3Key: prerenderManifest.routes[key].dataRoute.slice(1),
            filePath: pageFilePath,
        });
    });
    const prerenderManifestHTMLPageUploads = Object.keys(prerenderManifest.routes).map((key) => {
        const relativePageFilePath = key.endsWith('/')
            ? path_1.default.posix.join(key, 'index.html')
            : `${key}.html`;
        const pageFilePath = utils_1.pathToPosix(path_1.default.join(dotNextDirectory, `serverless/pages/${relativePageFilePath}`));
        return s3.uploadFile({
            s3Key: path_1.default.posix.join('static-pages', relativePageFilePath),
            filePath: pageFilePath,
            cacheControl: exports.SERVER_CACHE_CONTROL_HEADER,
        });
    });
    const uploadPublicOrStaticDirectory = async (directory, publicDirectoryCache) => {
        const directoryPath = path_1.default.join(nextStaticDir, directory);
        if (!(await fs_extra_1.pathExists(directoryPath))) {
            return Promise.resolve([]);
        }
        const files = await utils_1.readDirectoryFiles(directoryPath);
        return files.filter(utils_1.filterOutDirectories).map((fileItem) => s3.uploadFile({
            filePath: fileItem.path,
            s3Key: utils_1.pathToPosix(path_1.default.relative(path_1.default.resolve(nextStaticDir), fileItem.path)),
            cacheControl: getPublicAssetCacheControl_1.default(fileItem.path, publicDirectoryCache),
        }));
    };
    const publicDirUploads = await uploadPublicOrStaticDirectory('public', publicDirectoryCache);
    const staticDirUploads = await uploadPublicOrStaticDirectory('static', publicDirectoryCache);
    const allUploads = [
        ...buildStaticFileUploads,
        ...htmlPageUploads,
        ...prerenderManifestJSONPropFileUploads,
        ...prerenderManifestHTMLPageUploads,
        ...publicDirUploads,
        ...staticDirUploads,
    ];
    return Promise.all(allUploads);
};
exports.default = uploadStaticAssets;
