"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const config_1 = require("./config");
const context_1 = __importDefault(require("./context"));
const utils_1 = require("./utils");
const deploy = async (deployConfigPath, methodName = 'default') => {
    let options = {};
    try {
        options = await Promise.resolve().then(() => __importStar(require(deployConfigPath)));
    }
    catch (e) { }
    const { debug = false, engine = config_1.DEFAULT_ENGINE, onPreDeploy, onPostDeploy, onShutdown, stage, } = options;
    const engineIndex = config_1.SUPPORTED_ENGINES.findIndex(({ type }) => type === engine);
    const isInit = methodName === 'init';
    onShutdown && handleShutDown(onShutdown);
    if (engineIndex === -1) {
        console.error(`❌ ${chalk_1.default.red(`Unsupported engine:`)}: ${engine}\nPick one of: ${config_1.SUPPORTED_ENGINES.map(({ type }) => type).join(', ')}`);
        process.exit(1);
    }
    const EngineComponent = await Promise.resolve().then(() => __importStar(require(config_1.SUPPORTED_ENGINES[engineIndex].component)));
    const method = config_1.METHOD_NAME_MAP.find(({ name }) => name === methodName);
    if (!method) {
        console.error(`❌ ${chalk_1.default.red(`Unsupported method:`)} ${methodName}\nPick one of: ${config_1.METHOD_NAME_MAP.map(({ name }) => name).join(', ')}`);
        process.exit(1);
    }
    utils_1.createBaseConfig(deployConfigPath, isInit);
    if (isInit) {
        process.exit(0);
    }
    const context = new context_1.default({
        root: process.cwd(),
        stateRoot: path_1.default.join(process.cwd(), config_1.STATE_ROOT, (stage === null || stage === void 0 ? void 0 : stage.name) || 'local'),
        debug,
        entity: engine.toUpperCase(),
        message: method.action,
    });
    const component = new EngineComponent.default(undefined, context);
    try {
        context.log(`⚡ Starting ${method.actionNoun} ⚡`);
        onPreDeploy && (await onPreDeploy());
        await component.init();
        context.metrics.lastDebugTime = new Date().getTime();
        context.statusEngineStart();
        const outputs = await component[methodName](options);
        context.renderOutputs(outputs);
        onPostDeploy && (await onPostDeploy());
        context.close('done');
        process.exit(0);
    }
    catch (e) {
        context.close('error', e);
        process.exit(1);
    }
};
function handleShutDown(onShutdown) {
    const doShutdown = async () => {
        await onShutdown();
        process.exit(1);
    };
    process.on('SIGINT', doShutdown);
    process.on('SIGQUIT', doShutdown);
    process.on('SIGTERM', doShutdown);
}
exports.default = deploy;
