"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@serverless/core");
const path_1 = require("path");
const gh_pages_1 = require("gh-pages");
const fs_extra_1 = require("fs-extra");
const fs_1 = require("fs");
const builder_1 = __importDefault(require("./builder"));
class GithubComponent extends core_1.Component {
    async default(inputs = {}) {
        await this.build(inputs);
        return this.deploy(inputs);
    }
    async build({ build, nextConfigDir, domain } = {}) {
        this.context.status('Building');
        const nextConfigPath = nextConfigDir ? path_1.resolve(nextConfigDir) : process.cwd();
        await builder_1.default({
            cmd: (build === null || build === void 0 ? void 0 : build.cmd) || 'node_modules/.bin/next',
            cwd: (build === null || build === void 0 ? void 0 : build.cwd) ? path_1.resolve(build.cwd) : nextConfigPath,
            args: (build === null || build === void 0 ? void 0 : build.args) || ['build'],
        }, this.context.instance.debugMode ? this.context.debug : undefined);
        if (domain === null || domain === void 0 ? void 0 : domain.length) {
            const computedDomain = getComputedDomain(domain);
            if (computedDomain) {
                fs_1.writeFileSync('out/CNAME', computedDomain);
            }
        }
    }
    async deploy({ domain, publish: publishOptions } = {}) {
        this.context.status('Deploying');
        const outputs = {};
        if (domain === null || domain === void 0 ? void 0 : domain.length) {
            const computedDomain = getComputedDomain(domain);
            outputs.appUrl = `https://${computedDomain}`;
        }
        const publishPromise = new Promise((resolve, reject) => {
            gh_pages_1.publish('out', { message: 'Next Deployment Update', dotfiles: true, ...publishOptions }, (err) => {
                if (err) {
                    return reject(err);
                }
                resolve();
            });
        });
        await publishPromise;
        return outputs;
    }
    async remove() {
        await fs_extra_1.emptyDir('out');
        fs_1.writeFileSync('out/empty', '');
        const publishPromise = new Promise((resolve, reject) => {
            gh_pages_1.publish('out', { message: 'Next Deployment Removal', remove: '*' }, (err) => {
                if (err) {
                    return reject(err);
                }
                resolve();
            });
        });
        await publishPromise;
    }
}
function getComputedDomain(inputDomain) {
    let domain;
    if (typeof inputDomain === 'string') {
        domain = inputDomain;
    }
    else if (inputDomain instanceof Array) {
        domain = inputDomain.join();
    }
    return domain;
}
exports.default = GithubComponent;
